//**********************************************************************
// LifePanel Class File
// The panel that runs the simulation and interacts with the user.
//******************************************************

import java.util.Random;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class LifePanel extends JPanel
{

	final int  CELLS_ACROSS = 200;
	final int  CELLS_DOWN   = 100;
	
	final int CELL_SIZE = 4;
	
	final int SCROLLBAR_WIDTH = 12;
	
	final int BOTTOM_BAR_WIDTH = 20;
	final int RIGHT_BAR_WIDTH = 48;
	
	 int WINDOW_WIDTH = CELLS_ACROSS * CELL_SIZE + RIGHT_BAR_WIDTH + SCROLLBAR_WIDTH;
	 int WINDOW_HEIGHT = (CELLS_DOWN * CELL_SIZE) + BOTTOM_BAR_WIDTH + SCROLLBAR_WIDTH;
	

	LifePrefs myPrefs;
	
	int entireWidth;
	int entireHeight;
	int frameWidth;
	int frameHeight;
	
	int cellsAcrossFit;
	int cellsDownFit;
	
	int hScrollOffset;
	int hScrollWidth;
	int vScrollOffset;
	int vScrollWidth;
	int scrollPct;
	
	double hScale;
	double vScale;

	int delay;
	private Timer timer;
	boolean paused;
	private Random rand;
	public board dish;
	boolean drawGrid;
	int generation;
	int zoom;
	boolean splashScreen;
	
	LifeKeys keyboard;
	Scrollbar horiz, vert;
	
	boolean dragState;
	boolean dragScroll;
	Scrollbar scrolling;
	
	public LifePanel () 
	{
		myPrefs = new LifePrefs();
		//
		myPrefs.set(12,8,Color.black, Color.white,true);
		zoom = CELL_SIZE;
		paused = false;
		delay = 20;
		keyboard = new LifeKeys();
		generation = 0;
		dish = new board(CELLS_ACROSS, CELLS_DOWN);	
		dish.current = dish.loadArray("splash.dna");
		splashScreen = true;
		timer = new Timer(2500, new TimerListener()); // to slow down drawing
		addMouseListener (new ClickListener()); // for resetting
		addMouseMotionListener (new DragListener());
		addKeyListener (keyboard);

		setBackground(Color.black);
		setPreferredSize (new Dimension(WINDOW_WIDTH, WINDOW_HEIGHT));
		
		horiz = new Scrollbar	(0,WINDOW_HEIGHT-BOTTOM_BAR_WIDTH-SCROLLBAR_WIDTH,
										50,
										WINDOW_HEIGHT - BOTTOM_BAR_WIDTH, 
										0, WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH, true);
		
		vert = new Scrollbar (WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH, 0,
									WINDOW_WIDTH - RIGHT_BAR_WIDTH, 29, 0, 
									WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH, false);
		
		zoomAndScroll(0,0,4);

		
		rand = new Random(); // create a new Random object
		int i;
		this.setFocusable(true); 
		timer.start(); // start the timer
	}
	
	public void zoomAndScroll (int hNewScroll, int vNewScroll, int newZoom)
	{
		// calculate new Horizontal and Vertical Scrolls. If they are
		// greaters than the maximum scrollable amount, they will remain
		// the same
		int oldHScroll = dish.getHScroll();
		int oldVScroll = dish.getVScroll();
		int oldZoom = zoom;
		if (hNewScroll != -1)
		{
			if (hNewScroll < (dish.xSize - cellsAcrossFit)+1)
				dish.setHScroll(hNewScroll);
		}
		if (vNewScroll != -1)
		{
			if (vNewScroll < (dish.ySize - cellsDownFit)+1)
				dish.setVScroll(vNewScroll);
		}
		if (newZoom != 0)
			zoom = newZoom;
			
		// calculate the entire width of the whole petri dish, if there were
		// a monitor big enough to show it
		entireWidth = zoom * dish.xSize;
		entireHeight = zoom * dish.ySize;
		
		// calculate the width of the frame
		frameWidth = WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH;
		frameHeight = WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH;
		
		// calculate what percentage of the entire dish is shown in the frame
		hScale = (double)frameWidth / (double)entireWidth;
		vScale = (double) frameHeight / (double) entireHeight;
		
		
		
		
		// calculate how many cells fit across and down
		cellsAcrossFit = frameWidth / zoom;
		cellsDownFit = frameHeight / zoom;
		
		if (cellsAcrossFit > dish.xSize)
			dish.setHScroll(0);
		if (cellsDownFit > dish.ySize)
			dish.setVScroll(0);
		
	//	System.out.println(dish.xSize - cellsAcrossFit);
		int overlap = cellsAcrossFit - (dish.xSize - dish.getHScroll());
		if (overlap > 1 && zoom > 4)
		{
			dish.setHScroll(dish.getHScroll() - (cellsAcrossFit - (dish.xSize - dish.getHScroll())));
		}

		overlap = cellsDownFit - (dish.ySize - dish.getVScroll());
		if (overlap > 1 && zoom > 4)
		{
			dish.setVScroll(dish.getVScroll() - (cellsDownFit - (dish.ySize - dish.getVScroll())));
		}
		/*

		if (dish.getVScroll() > (dish.ySize - cellsDownFit))
		{
			dish.setVScroll(dish.getVScroll() - ((dish.ySize - cellsDownFit)));
		}
		*/
		// calculate the sizes of the scrollbars and their offsets
		hScrollWidth = (int)((double)hScale * (double)(WINDOW_WIDTH - RIGHT_BAR_WIDTH));
		vScrollWidth = (int)((double)vScale * (double)(WINDOW_HEIGHT - BOTTOM_BAR_WIDTH));
		
		hScrollOffset = (int)((double)hScale * (double)dish.hScroll * (double)zoom);
		vScrollOffset = (int)((double)vScale * (double)dish.vScroll * (double)zoom);
		
		if (hScrollWidth + hScrollOffset > frameWidth)
			hScrollWidth = frameWidth - hScrollOffset;
		
		if (vScrollWidth + vScrollOffset > frameHeight)
			vScrollWidth = frameHeight - vScrollOffset;
		
		// calculate what percent the Zoom: label displays
		scrollPct = zoom * 100 / 4;
		
		
		horiz.resize(hScrollOffset, hScrollOffset+hScrollWidth);
		vert.resize(vScrollOffset, vScrollOffset+vScrollWidth);
		//horiz.l = hScrollOffset;
		//horiz.r = hScrollOffset + hScrollWidth;

	}
	
	public int lesser (int num1,int num2)
	{
		if (num1 < num2)
			return num1;
		else return num2;
	}
	
	public void paintComponent (Graphics page)
	{	
		super.paintComponent(page);
		setBackground(myPrefs.backColor);

		if (WINDOW_WIDTH != getWidth() || WINDOW_HEIGHT != getHeight())
		{
			WINDOW_WIDTH = getWidth();
			WINDOW_HEIGHT = getHeight();
			horiz.resizeTrack(0,WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH,
									WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH,
									WINDOW_HEIGHT - BOTTOM_BAR_WIDTH);
			vert.resizeTrack(WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH, 0,
									WINDOW_WIDTH - RIGHT_BAR_WIDTH, 
									WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH);
			zoomAndScroll(-1,-1,0);
		}
		// paint the dish and its 'bounds box'
		page.setColor(Color.white);
		page.drawRect(-1,-1,zoom*dish.xSize - (dish.hScroll*zoom) + 1,
										zoom*dish.ySize - (dish.vScroll * zoom)+ 1);
		if (myPrefs.showAge)
			dish.paintAgedGeneration(page, zoom, drawGrid, myPrefs);	
		else
			dish.paintGeneration(page, zoom, drawGrid, myPrefs);		
		
		/*
		// the Scrollbar Tracks
		page.setColor(Color.cyan);
		page.fillRect(0,WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH,
							WINDOW_WIDTH - RIGHT_BAR_WIDTH+SCROLLBAR_WIDTH, 12);
		page.fillRect(WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH, 0, 12, 
							WINDOW_HEIGHT - BOTTOM_BAR_WIDTH);
		*/
		// the Scrollbars
		/*
		page.setColor(Color.gray);
		page.fillRect(hScrollOffset,WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH,
									 hScrollWidth, SCROLLBAR_WIDTH);
		page.fillRect(WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH, vScrollOffset, 
								12, vScrollWidth);
		*/				
		//if (dragScroll)
		horiz.draw(page);
		vert.draw(page);
								
		// the Button Bars
		page.setColor(new Color(100,100,100));
		page.fillRect(0,WINDOW_HEIGHT - BOTTOM_BAR_WIDTH, 
								WINDOW_WIDTH, BOTTOM_BAR_WIDTH);
		page.fillRect(WINDOW_WIDTH - RIGHT_BAR_WIDTH,0, 
								RIGHT_BAR_WIDTH, WINDOW_HEIGHT);	
	
		// the Labels
		page.setColor(Color.white);
		page.drawString ("Generation: " + generation, 20, WINDOW_HEIGHT - 5);
		page.drawString ("Population: " + dish.population, 170, WINDOW_HEIGHT - 5);
		page.drawString("Zoom: " + scrollPct + "%", 280, WINDOW_HEIGHT - 5);
		
		
	}
	
	
	class TimerListener implements ActionListener
	{
		public void actionPerformed (ActionEvent event)
		{
			if (splashScreen)
			{
				timer.setDelay(delay);
				splashScreen = false;
			}
			if (!paused)
			{
				if (myPrefs.torus)
					dish.runTorusGeneration(generation);
				else
					dish.runGeneration(generation);
				generation++;
				repaint();
			}
		}
	}
	
	class Scrollbar
	{
		boolean orientation;
		boolean isDragging;
		boolean enabled;
		
		final boolean HORIZONTAL = true;
		final boolean VERTICAL = false;
		
		int t, l, b, r;
		int width;
		int min, max;
		int offset;
		int scrollZoom;
		
		public boolean horizontal ()
		{
			return HORIZONTAL;
		}
		
		public boolean vertical ()
		{
			return VERTICAL;
		}
		
		public boolean isInMe (MouseEvent e)
		{
			int x = e.getX();
			int y = e.getY();
			return ((x > l && y > t && x < r && y < b) && enabled);
		}

		
		public Scrollbar (int left, int top, int right, int bottom, 
								int initMin, int initMax, boolean orient)
		{
			t = top;
			l = left;
			b = bottom;
			r = right;
			
			min = initMin;
			max = initMax;
			
			
			orientation = orient;
			
			if (orientation == HORIZONTAL)
				width = r - l;
			else
				width = b - t;
			
			isDragging = false;
		}
		
		public boolean isEnabled()
		{
			return enabled;
		}
		
		public void resize (int start, int end)
		{
			if (orientation == HORIZONTAL)
			{
				enabled = !((cellsAcrossFit >= dish.getXSize()));
				l = start;
				r = end;
				width = r - l;
				
			}
			else
			{
				enabled = !((cellsDownFit >= dish.getYSize()));
				t = start;
				b = end;
				width = b - t;
			}
		}
		
		public void resizeTrack (int left, int top, int right, int bottom)
		{
			if (orientation == HORIZONTAL)
			{
				min = left;
				max = right;
				t = top;
				b = bottom;
			}
			else
			{
				min = top;
				max = bottom;
				l = left;
				r = right;
			}
		}
		
		public void startDrag (MouseEvent e)
		{
			isDragging = true;
			if (orientation == HORIZONTAL)
				offset = l - e.getX();
			else
				offset = t - e.getY();
		}
		
		public void endDrag ()
		{
			isDragging = false;
		}
		
		public void draw (Graphics g)
		{
			Color myColor;
			g.setColor(Color.cyan);
			if (orientation == HORIZONTAL)
				g.fillRect(min,t,(max-min)+SCROLLBAR_WIDTH,SCROLLBAR_WIDTH);
			else
				g.fillRect(l,min,SCROLLBAR_WIDTH,(max-min)+SCROLLBAR_WIDTH);			
			if (isDragging)
				myColor = new Color(175,175,255);
			else
				myColor = new Color(150,150,255);
			if (!(enabled))
				myColor = new Color(220,220,255);
			g.setColor(myColor);
			g.fillRect(l,t,(r-l), (b-t));
			g.setColor(Color.black);
			g.drawRect(l+1,t+1,(r-l)-3,(b-t)-3);
		}
		
		public int drag (MouseEvent e)
		{
			double pct = 0.0;
			int x = e.getX();
			int y = e.getY();
			if (orientation == HORIZONTAL)
			{
				l = x + offset;
				r = l + width;
				//r = l + x + offset;
				if (l < min)
				{
					l = min;
					r = l + width;
				}
				if ( r > max)
				{
					r = max;
					l = r - width;
					return dish.getXSize() - cellsAcrossFit;
				}
				return ((l - min) * dish.getXSize() / (max - min));
			}
			else
			{
				t = y + offset;
				b = t + width;
				if (t < min)
				{
					t = min;
					b = t + width;
				}
				if (b > max)
				{
					b = max;
					t = b - width;
					return dish.getYSize() - cellsDownFit;
				}
				return ((t - min) * dish.getYSize() / (max - min));



			}
		}
	}
	
	class DragListener extends MouseMotionAdapter
	{
		public void mouseDragged(MouseEvent event)
		{
			if (dragScroll)
			{
				if (scrolling == horiz)
					dish.setHScroll(scrolling.drag(event));
				else
					dish.setVScroll(scrolling.drag(event));
				repaint();
			}
			else
			{
				int x = event.getX();
				int y = event.getY();
			
		
				if ((y > WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH) &&
					(x > WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH))
				{
				//	paused = !(paused);
				}
				else
				{
					x += (dish.getHScroll() * zoom);
					y += (dish.getVScroll() * zoom);
					x /= zoom;
					y /= zoom;
					//dragState = dish.getCellState(x,y);
					dish.setCell(x,y, dragState, generation);
					repaint();
				}
			}

		}
		
		public void mouseMoved (MouseEvent e)
		{
		}
	}
	
	public boolean clickedInBox(int x, int y)
	{
		int rightEdge = (dish.xSize * zoom) - (dish.hScroll*zoom);
		int bottomEdge = (dish.ySize * zoom) - (dish.vScroll*zoom);
		return (x < rightEdge && y < bottomEdge);
	}
	
	
	class ClickListener implements MouseListener
	{
		boolean dragSetting;
		
		// if the mouse is pressed, then re-randomize the bars
		public void mouseClicked (MouseEvent event)
		{
			int x = event.getX();
			int y = event.getY();
			if ((y > WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH) ||
				(x > WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH))
			{
				paused = !(paused);
				myPrefs.llama = !(myPrefs.llama);
			}
			else 
			{
				if (clickedInBox(x,y))
				{
				x += (dish.getHScroll() * zoom);
				y += (dish.getVScroll() * zoom);
				x /= zoom;
				y /= zoom;
	
					//dragState = !(dish.getCellState(x,y));
					dish.toggleCell(x,y, generation);
					repaint();
				}
			}

		}
		
		public void mousePressed(MouseEvent event) 
		{
			int x = event.getX();
			int y = event.getY();
			
			// if clicked in main window
			if ((y > WINDOW_HEIGHT - BOTTOM_BAR_WIDTH - SCROLLBAR_WIDTH) ||
				(x > WINDOW_WIDTH - RIGHT_BAR_WIDTH - SCROLLBAR_WIDTH))
			{
				if (horiz.isInMe(event))
				{
					dragScroll = true;
					horiz.startDrag(event);
					scrolling = horiz;
				}
				if (vert.isInMe(event))
				{
					dragScroll = true;
					vert.startDrag(event);
					scrolling = vert;
				}
			}
			else
			{
				x += (dish.getHScroll() * zoom);
				y += (dish.getVScroll() * zoom);
				x /= zoom;
				y /= zoom;
				dragState = !(dish.getCellState(x,y));
				repaint();
			}
		}
		
		public void mouseReleased(MouseEvent event)
		{
			if (dragScroll)
			{
				dragScroll = false;
				scrolling.endDrag();			
			}
		}
		public void mouseEntered(MouseEvent event) {}
		public void mouseExited(MouseEvent event) {}
	}

	public int SubToMin(int num1, int num2, int min)
	{
		if ((num1 - num2) < min)
			return min;
		else return (num1 - num2);
	}
	
	public int AddToMax(int num1, int num2, int max)
	{
		if (num1 + num2 > max)
			return max;
		else return (num1 + num2);
	}
	
	public void doDelete()
	{
		dish.clearBoard();
		repaint();
	}
	
	public void randomize()
	{
		String message = "What percent of the cells should be live?\n" +
								"(Enter a decimal between 0 and 1)";
		
		String text = JOptionPane.showInputDialog(null,message);
		if (text == null)
		{     
		}
		else
		{
				try
				{
					double d = Double.valueOf(text).doubleValue();
					dish.randomize(d, generation);
					repaint();
				}
				catch (NumberFormatException ex)
				{
					 String nag = "Error: this is not a decimal between 0 and 1.";
					 JOptionPane.showMessageDialog(null, nag);
				}
		}
	}
	
	public int getGeneration()
	{
		return generation;
	}
	
	public void toggleGrid()
	{
		drawGrid = !(drawGrid);
		repaint();
	}
	
	public void changeZoom (int amount)
	{
		if (amount < 0)
		{
			if (zoom > 1)
				zoom += amount;
		}
		else
		{
			zoom += amount;
		}
		repaint();
	}
	
	public void zoomTo (int amount)
	{
		if (amount > 0)
			zoom = amount;
		repaint();
	}

 class LifeKeys implements KeyListener
	{
		public void keyTyped(KeyEvent k) {}
		
		public void keyPressed(KeyEvent k)
		{
			switch (k.getKeyCode())
			{
				case KeyEvent.VK_DELETE:
				{
					doDelete();
					break;
				}
				case KeyEvent.VK_2:
				{
					dish.randomize(2, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_3:
				{
					dish.randomize(3, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_4:
				{
					dish.randomize(4, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_5:
				{
					dish.randomize(5, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_6:
				{
					dish.randomize(6, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_7:
				{
					dish.randomize(7, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_8:
				{
					dish.randomize(8, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_9:
				{
					dish.randomize(9, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_0:
				{
					dish.randomize(10, generation);
					generation = 0;
					repaint();
					break;
				}
				case KeyEvent.VK_SPACE:
				{
					paused = !(paused);
					break;
				}
				case KeyEvent.VK_UP:
				{
					if (delay > 0) delay -= 10;
					timer.setDelay(delay);
					break;
				}	
				case KeyEvent.VK_DOWN:
				{
					delay += 10;
					timer.setDelay(delay);
					break;
				}	
				case KeyEvent.VK_RIGHT:
				{
					zoomAndScroll(-1,-1, zoom + 1);
					repaint();
					break;
				}
				case KeyEvent.VK_LEFT:
				{
					zoomAndScroll(-1,-1, zoom - 1);
					repaint();
					break;
				}
				case KeyEvent.VK_NUMPAD6:
				{
					//dish.scrollHorizontal(1);
					
					zoomAndScroll(dish.getHScroll() +1, dish.getVScroll(), 0);
					repaint();
					break;
				}
				case KeyEvent.VK_NUMPAD4:
				{
					zoomAndScroll(dish.getHScroll() - 1, dish.getVScroll(), 0);
					repaint();
					break;
				}
				case KeyEvent.VK_NUMPAD2:
				{
					zoomAndScroll(dish.getHScroll(), dish.getVScroll() + 1, 0);
					repaint();
					break;
				}
				case KeyEvent.VK_NUMPAD8:
				{
					zoomAndScroll(dish.getHScroll(), dish.getVScroll() - 1, 0);;
					repaint();
					break;
				}
				case KeyEvent.VK_NUMPAD5:
				{
					zoomAndScroll(0,0, CELL_SIZE);
					//dish.setHScroll(0);
					//dish.setVScroll(0);
					//zoom = CELL_SIZE;
					repaint();
					break;
				}
				case KeyEvent.VK_G:
				{
					toggleGrid();
					break;
				}
				case KeyEvent.VK_SLASH:
				{
					if (myPrefs.torus)
						dish.runTorusGeneration(generation);
					else dish.runGeneration(generation);
					generation++;
					repaint();
					break;
				}
				case KeyEvent.VK_INSERT:
				{
					dish = new board(dish, dish.getXSize()-10, dish.getYSize());
					repaint();
					break;
				}
				case KeyEvent.VK_HOME:
				{
					dish = new board(dish, dish.getXSize()+10, dish.getYSize());
					repaint();
					break;
				}
				case KeyEvent.VK_PAGE_DOWN:
				{
					dish = new board(dish, dish.getXSize(), dish.getYSize()+10);
					repaint();
					break;
				}
				case KeyEvent.VK_PAGE_UP:
				{
					dish = new board(dish, dish.getXSize(), dish.getYSize()-10);
					repaint();
					break;
				}
				case KeyEvent.VK_S:
				{
					saveFile();
					break;
				}
				case KeyEvent.VK_O:
				{
					openFile();
					break;
				}
			}
		}
		public void keyReleased(KeyEvent k) {}

	}
	
	public void saveFile()
	{
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new dnaFileFilter());
		int ret = chooser.showSaveDialog(null);
		if (ret != JFileChooser.APPROVE_OPTION)
		{
		}
		else
		{
			File file = chooser.getSelectedFile();
			String fName = file.getPath();
			dish.writeToFile(fName);
		}
	}
	
	public void openFile ()
	{
		JFileChooser chooser = new JFileChooser();
		chooser.setAcceptAllFileFilterUsed(false);
		chooser.addChoosableFileFilter(new dnaFileFilter());
		int ret = chooser.showOpenDialog(null);
		if (ret != JFileChooser.APPROVE_OPTION)
		{
		}
		else
		{
			File file = chooser.getSelectedFile();
			String fName = file.getPath();
			dish = dish.readFromFile(fName, false);
			paused = true;
			repaint();
		}
	}

}