//**********************************************************************
// Board Class File
// A board holds an array of cells and the methods for acting
// on them.
//******************************************************

import java.util.Random;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class board implements Serializable
{
	cell[][] current;
	cell[][] next;
	int xSize;
	int ySize;
	boolean wraparound;
	Random myRand = new Random();
	
	public Color baseColor;
	
	final Color DEFAULT_COLOR = Color.white;
	
	int hScroll;
	int vScroll;
	
	int red;
	int green;
	int blue;
	
	boolean[] liveCellActions;
	boolean[] deadCellActions;
	int liveRules;
	int deadRules;
	
	final int LIVE_DEFAULT = 12;
	final int DEAD_DEFAULT = 8;
	
	int population;
	
	public void initColors ()
	{
		red = baseColor.getRed();
		green = baseColor.getGreen();
		blue = baseColor.getBlue();
	}
	
	public void setCellActions (int liveSeed, int deadSeed)
	{
		if (liveSeed < 0 || deadSeed < 0)
		{
			liveSeed = liveRules = LIVE_DEFAULT;
			deadSeed = deadRules = DEAD_DEFAULT;
		}
		liveCellActions = new boolean[9];
		deadCellActions = new boolean[9];
		int bits, placeCount;
		
		// calculate live actions
		bits = liveSeed;
		placeCount = 0;
		while (bits > 0)
		{
			liveCellActions[placeCount] = ((bits & 1) == 1);
			placeCount++;
			bits = bits >> 1;
		}
		bits = deadSeed;
		placeCount = 0;
		while (bits > 0)
		{
			deadCellActions[placeCount] = ((bits & 1) == 1);
			placeCount++;
			bits = bits >> 1;
		}	
	}
	
	public board (int initXSize, int initYSize)
	{
		setCellActions(-1,-1);
		baseColor = DEFAULT_COLOR;
		initColors();
		hScroll = 0;
		vScroll = 0;
		population = 0;
		xSize = initXSize;
		ySize = initYSize;
		wraparound = false;
		current = new cell[ySize][xSize];
		next = new cell[ySize][xSize];
		int r, c;
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				current[r][c] = new cell();
				next[r][c] = new cell();
			}
		}
	}
	
	public board (board oldBoard, int initXSize, int initYSize)
	{
		setCellActions(-1,-1);
		baseColor = oldBoard.baseColor;
		initColors();
		hScroll = oldBoard.hScroll;
		vScroll = oldBoard.vScroll;
		population = 0;
		xSize = initXSize;
		ySize = initYSize;
		wraparound = oldBoard.wraparound;
		current = new cell[ySize][xSize];
		cell temp = new cell();
		next = new cell[ySize][xSize];
		
		int r, c;
		
		for (r = 0; r < ySize; r++)
			for (c = 0; c < xSize; c++)
			{
				current[r][c] = new cell();
				next[r][c] = new cell();
				if ((r < oldBoard.ySize) && (c < oldBoard.xSize))
				{
					//temp.copy(oldBoard.current[r][c]);
					current[r][c].copy(oldBoard.current[r][c]);
					if (current[r][c].isAlive())
						population++;
				}
			}
	}
	
	public board (cell[][] dish)
	{
		baseColor = DEFAULT_COLOR;
		initColors();
		setCellActions(-1,-1);
		int r, c;
		hScroll = 0;
		vScroll = 0;
		population = 0;
		ySize = dish.length;
		xSize = dish[0].length;
		current = new cell[ySize][xSize];
		next = new cell[ySize][xSize];
		wraparound = false;
		for (r = 0; r < ySize; r++)
			for (c = 0; c < xSize; c++)
			{
				current[r][c] = new cell();
				next[r][c] = new cell();
				current[r][c].copy(dish[r][c]);
				if (current[r][c].isAlive())
					population++;
			}
	}
	
	public int getXSize ()
	{
		return xSize;
	}
	
		public int getYSize ()
	{
		return ySize;
	}
	
	public void writeToFile(String fileName)
	{
		saveArray(fileName, current);
	}
	
	public board readFromFile(String fileName, boolean relative)
	{
		board temp;
		temp = new board(loadArray(fileName, relative));
		return temp;
	}
	
	
	public void saveArray(String filename, cell[][] output_veld) {
     try {
       FileOutputStream fos = new FileOutputStream(filename);
        ObjectOutputStream out = new ObjectOutputStream(fos);
        out.writeObject(output_veld);
        out.flush();
        out.close();
     }
     catch (IOException e) {
         System.out.println(e); 
     }
  }

	public cell[][] loadArray(String filename) {
      try {
			ClassLoader loader = this.getClass().getClassLoader();
			InputStream stream = loader.getResourceAsStream(filename);
        ObjectInputStream in = new ObjectInputStream(stream);
        cell[][] gelezen_veld = (cell[][])in.readObject();
        in.close();
        return gelezen_veld;
      }
      catch (Exception e) {
		System.out.println(e.getMessage());
			e.printStackTrace();
          System.out.println(e);
      }
      return null;
  }
  
  public cell[][] loadArray (String filename, boolean relative)
  {
  		if (relative)
			return loadArray(filename);
		
  		try {
			FileInputStream stream = new FileInputStream(filename);
        ObjectInputStream in = new ObjectInputStream(stream);
        cell[][] gelezen_veld = (cell[][])in.readObject();
        in.close();
        return gelezen_veld;
      }
      catch (Exception e) {
		System.out.println(e.getMessage());
			e.printStackTrace();
          System.out.println(e);
      }
      return null;	
  }

	public void flush ()
	{
		int r, c;
		for (r = 0; r < ySize; r++)
			for (c = 0; c < xSize; c++)
			{
				current[r][c].kill();
				next[r][c].kill();
			}
	}
	
	// randomize
	// double chance -- the chance, as a decimal, that a cell will be alive
	// int curGen -- the 'birth generation' for the new live cells
	public void randomize (double chance, int curGen)
	{
		// create a new Random object
		Random rand = new Random();
		int r, c; // for stepping through the cell array
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				// since the chance is chance of a  cell being alive, that
				// means a random number less than or equal to chance creates
				// a live cell.
				if (rand.nextDouble() <= chance)
					current[r][c].makeAlive(curGen);
				else
					current[r][c].kill();
			}
		}
	}

	
	public void randomize(int chance, int curGen)
	{
		Random rand = new Random();
		int r, c;
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				if (rand.nextInt(chance) == 1)
					current[r][c].makeAlive(curGen);
				else
					current[r][c].kill();
			}
		}
	}
	
	
	
	public int subToMin (int num1, int num2, int min)
	{
		if (num1 - num2 < min)
			return min;
		else return (num1 - num2);
	}
	
	public void scrollHorizontal (int amount)
	{
		if (amount < 0)
		{
			if (hScroll > 0)
			{
				hScroll += amount;
			}
		}
		else
			hScroll += amount;
	}
	
	public void scrollVertical (int amount)
	{
		if (amount < 0)
		{
			if (vScroll > 0)
			{
				vScroll += amount;
			}
		}
		else
			vScroll += amount;
	}
	
	public void setHScroll (int amount)
	{
		hScroll = amount;
	}
	
	public void setVScroll (int amount)
	{
		vScroll = amount;
	}
	
	public int getHScroll ()
	{
		return hScroll;
	}
	
	public int getVScroll ()
	{
		return vScroll;
	}
	
	public void clearBoard()
	{
		int r, c;
		for (r = 0; r < ySize; r++)
			for (c = 0; c < xSize; c++)
			{
				current[r][c].kill();
				next[r][c].kill();
			}
	}
	
	public void runGeneration(int curGen)
	{
		int r, c;
		int neighbors = 0;
		population = 0;
		Random rand = new Random();
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				neighbors = 0;
				if (r > 0 && c > 0)
					if (current[r-1][c-1].isAlive()) neighbors++;
			
				if (r > 0)
					if (current[r-1][c].isAlive()) neighbors++;	
				
				if (r > 0 && c < xSize-1)
					if (current[r-1][c+1].isAlive()) neighbors++;	

				if (c > 0)
					if (current[r][c-1].isAlive()) neighbors++;	

				if (c < xSize-1)
					if (current[r][c+1].isAlive()) neighbors++;	
					
				if (r < ySize-1 && c > 0)
					if (current[r+1][c-1].isAlive()) neighbors++;
			
				if (r < ySize-1)
					if (current[r+1][c].isAlive()) neighbors++;	
				
				if (r < ySize-1 && c < xSize-1)
					if (current[r+1][c+1].isAlive()) neighbors++;	


			//	if (neighbors == 1)
			//		next[r][c].makeAlive(curGen);
			//	if (neighbors > 3)
			//		next[r][c].kill();
			
				if (current[r][c].isAlive())
				{
					if (liveCellActions[neighbors])
					{
						next[r][c].copy(current[r][c]);
						next[r][c].genUp();
					}
					else
						next[r][c].kill();
				}
				else
				{
					if (deadCellActions[neighbors])
					{
						next[r][c].makeAlive(curGen);
					}
					else
					{
						next[r][c].kill();
					}
				}
				/*
			
				if (current[r][c].isAlive() && (neighbors < 2 || neighbors > 3))
					next[r][c].kill();
				else if (current[r][c].isAlive())
				{
					next[r][c].copy(current[r][c]);
					next[r][c].genUp();
				}
				
			//	if (current[r][c].getGen() > 200)
			//		next[r][c].kill();
				
				if (current[r][c].isDead() && neighbors == 3)
					next[r][c].makeAlive(curGen);
				*/
				
			}
		}
		
		for (r = 0; r < ySize; r++)
			for (c = 0; c < xSize; c++)
			{
				current[r][c].copy(next[r][c]);
				if (current[r][c].isAlive())
				{
					population++;
				}
			}
			//	current[r][c].setLife(next[r][c].isAlive());			
	}
	
	public void db (String s)
	{
		System.out.println(s);
	}
	
	public void runTorusGeneration(int curGen)
	{
		int r, c;
		int neighbors = 0;
		population = 0;
		
		cell nTouch = new cell();
		cell nwTouch = new cell();
		cell wTouch = new cell();
		cell swTouch = new cell();
		cell sTouch = new cell();
		cell seTouch = new cell();
		cell eTouch = new cell();
		cell neTouch = new cell();
		
		Random rand = new Random();
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				neighbors = 0;
				
				 nTouch = null;
				 nwTouch = null;
				 wTouch = null;
				 swTouch = null;
				 sTouch = null;
				 seTouch = null;
				 eTouch = null;
				 neTouch = null;

				
				if (r == 0 && c == 0)
					nwTouch = current[ySize - 1][xSize -1 ];
				else if (r == 0 && c > 0)
					nwTouch = current[ySize - 1][c-1];
				else if (r > 0 && c == 0)
					nwTouch = current[r - 1][xSize - 1];
				else
					nwTouch = current[r-1][c-1];
					
				if (r == 0)
					nTouch = current[ySize - 1][c];
				else
					nTouch = current[r-1][c];
					
				if (r == 0 && c == xSize - 1)
					neTouch = current[ySize - 1][0];
				else if (r == 0 && c < xSize - 1)
					neTouch = current[ySize - 1][c + 1];
				else if (r > 0 && c == xSize - 1)
					neTouch = current[r-1][0];
				else
					neTouch = current[r-1][c+1];
					
				if (c == 0)
					wTouch = current[r][xSize-1];
				else
					wTouch = current[r][c-1];
					
				if (c == xSize - 1)
					eTouch = current[r][0];
				else
					eTouch = current[r][c+1];
				
				
				if (r == ySize - 1 && c == 0)
					swTouch = current[0][xSize - 1];
				else if (r == ySize - 1 && c > 0)
					swTouch = current[0][c-1];
				else if (r < ySize - 1 && c == 0)
					swTouch = current[r+1][xSize -1];
				else 
					swTouch = current[r+1][c-1];
					
				
													
				if (r == ySize - 1)
					sTouch = current[0][c];
				else
					sTouch = current[r+1][c];
					
				if (r == ySize - 1 && c == xSize - 1)
					seTouch = current[0][0];
				else if (r == ySize - 1 && c < xSize - 1)
					seTouch = current[0][c+1];
				else if (r < ySize - 1 && c == xSize - 1)
					seTouch = current[r+1][0];
				else
					seTouch = current[r+1][c+1];
							
				if (nwTouch.isAlive())
					neighbors++;
					
				if (nTouch.isAlive())
					neighbors++;
				
				if (neTouch.isAlive())
					neighbors++;
					
				if (wTouch.isAlive())
					neighbors++;
					
				if (eTouch.isAlive())
					neighbors++;
					
				if (swTouch.isAlive())
					neighbors++;
			
				if (sTouch.isAlive())
					neighbors++;
					
				if (seTouch.isAlive())
					neighbors++;
					
			//	if (neighbors == 1)
			//		next[r][c].makeAlive(curGen);
			//	if (neighbors > 3)
			//		next[r][c].kill();
			
				if (current[r][c].isAlive())
				{
					if (liveCellActions[neighbors])
					{
						next[r][c].copy(current[r][c]);
						next[r][c].genUp();
					}
					else
						next[r][c].kill();
				}
				else
				{
					if (deadCellActions[neighbors])
					{
						next[r][c].makeAlive(curGen);
					}
					else
					{
						next[r][c].kill();
					}
				}
				/*
			
				if (current[r][c].isAlive() && (neighbors < 2 || neighbors > 3))
					next[r][c].kill();
				else if (current[r][c].isAlive())
				{
					next[r][c].copy(current[r][c]);
					next[r][c].genUp();
				}
				
			//	if (current[r][c].getGen() > 200)
			//		next[r][c].kill();
				
				if (current[r][c].isDead() && neighbors == 3)
					next[r][c].makeAlive(curGen);
				*/
				
			}
		}
		
		for (r = 0; r < ySize; r++)
			for (c = 0; c < xSize; c++)
			{
				current[r][c].copy(next[r][c]);
				if (current[r][c].isAlive())
				{
					population++;
				}
			}
			//	current[r][c].setLife(next[r][c].isAlive());			
	}

	
	public int greater(int num1, int num2)
	{
		if (num1 > num2)
			return num1;
		else return num1;
	}
	
	
	public int subColor(int init, int sub)
	{
		int ret = init;
		ret = init - sub;
		if (ret < 20)
			ret = 20;
		return ret;
	}
	
	public void paintGeneration (Graphics page, int cellSize, boolean drawGrid, LifePrefs myPrefs)
	{
		int r, c, g, t, l;
		int drawSize = 0;
		
		if (drawGrid)
			if (cellSize > 2)
			{
				page.setColor(new Color(75,75,75));
				{
				for (r = 0; r < ySize; r++)
				{
					page.drawLine(0, (r * cellSize) -1, cellSize * xSize, (r * cellSize)-1);
				}
				for (c = 0; c < xSize; c++)
				{
					page.drawLine((c * cellSize) - 1, 0, (c * cellSize) - 1, cellSize * ySize);
				}
			}
		}
		
		drawSize = cellSize;
		if (drawSize > 3)
			drawSize--;
		
		page.setColor(baseColor);
		
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				if (current[r][c].isAlive())
				{
						t = (cellSize * r) - (vScroll * cellSize);
						l = (cellSize * c) - (hScroll * cellSize);
						page.fillRect(l,t,drawSize,drawSize);
				}
			}
		}
	}
	
	public void paintAgedGeneration (Graphics page, int cellSize, boolean drawGrid, LifePrefs myPrefs)
	{
		int r, c, g, t, l, drawSize;
		if (drawGrid)
			if (cellSize > 2)
			{
				page.setColor(new Color(75,75,75));
				{
				for (r = 0; r < ySize; r++)
				{
					page.drawLine(0, (r * cellSize) -1, cellSize * xSize, (r * cellSize)-1);
				}
				for (c = 0; c < xSize; c++)
				{
					page.drawLine((c * cellSize) - 1, 0, (c * cellSize) - 1, cellSize * ySize);
				}
			}
		}
		
		drawSize = cellSize;
		if (drawSize > 3)
			drawSize--;
		
		for (r = 0; r < ySize; r++)
		{
			for (c = 0; c < xSize; c++)
			{
				if (current[r][c].isAlive())
				{
					g = current[r][c].getGen() / 3;
					if (g > 220)
						g = 220;

					t = (cellSize * r) - (vScroll * cellSize);
					l = (cellSize * c) - (hScroll * cellSize);
					
					page.setColor(new Color(subColor(red,g),subColor(green,g),subColor(blue,g)));
					
					if (myPrefs.llama)
					{
						page.setColor(new Color(myRand.nextInt(255), 
											myRand.nextInt(255), myRand.nextInt(255)));
					}
										
					//if (cellSize > 3)
					//	page.fillRect(l,t,cellSize-1,cellSize-1);
				//	else
						page.fillRect(l,t,drawSize,drawSize);



					/*
					if (cellSize > 3)
						page.fillRect(l,t,cellSize-1,cellSize-1);
					else
						page.fillRect(l,t,cellSize,cellSize);
					*/
				}
			}
		}
	}

	
	public void print ()
	{
		int r, c;
		for (r = 0; r < ySize; r++)
		{
			System.out.print("\n");
			for (c = 0; c < xSize; c++)
			{
				current[r][c].printMe();
			}
		}
	}
	
	public void setCell (int x, int y, boolean state, int curGen)
	{
		if ((x < xSize && y < ySize) && (x > -1 && y > -1))
		{
			if (state)
			{
				current[y][x].makeAlive(curGen);
				next[y][x].makeAlive(curGen);
			}
			else
			{
				current[y][x].kill();
				next[y][x].kill();
			}
		}
	}
	
	public boolean getCellState(int x, int y)
	{
		if (x < xSize && y < ySize)
			return current[y][x].isAlive();
		else return false;
	}
	
	public void toggleCell (int x, int y, int curGen)
	{
		if ((x < xSize && y < ySize) && (x > -1 && y > -1))
		{
			if (current[y][x].isAlive())
			{
				population--;
				current[y][x].kill();
				next[y][x].kill();
			}
			else
			{
				population++;
				current[y][x].makeAlive(curGen);
				next[y][x].makeAlive(curGen);
			}
		}
	}
}