//**********************************************************************
// Cell Class File
// A cell is the basic building block of Life. It can either be on or
// off.
//*******************************************************************

import java.io.*; // for Serializable

public class cell implements Serializable
	{
		boolean live;	// whether or not the cell is alive
		int gen;			// the cell's age
		int startGen;	// the generation the cell was born in
		
		// Constructor. By default, a cell is dead
		public cell ()
		{
			live = false;
			gen = 0;
		}
		
		// copy
		// copies the cell in to the calling cell
		public void copy (cell in)
		{
			gen = in.gen;
			live = in.live;
			startGen = in.startGen;
		}
		
		// makeAlive ()
		// int curGen -- the current generation
		// sets live to true, and the cell age to 0
		public void makeAlive (int curGen)
		{
			live = true;
			gen = 0;
			startGen = curGen;
		}
		
		// kill ()
		// kills a cell
		public void kill ()
		{
			live = false;
			gen = 0;
			startGen = 0;
		}
		
		// isAlive ()
		// returns true if the cell is alive, false otherwise
		public boolean isAlive ()
		{
			return live;
		}
		
		// isDead ()
		// returns true if the cell is dead, false otherwise
		public boolean isDead ()
		{
			return !(live);
		}
		
		// getStartGen()
		// returns an int representing the cell's 'birth generation'
		public int getStartGen ()
		{
			return startGen;
		}
		
		// setStartGen ()
		// int newStartGen -- the new birth generation for the cell
		// sets the cell's birth generation
		public void setStartGen (int newStartGen)
		{
			startGen = newStartGen;
		}
		
		// setLife
		// boolean in -- the state to set the cell to
		public void setLife(boolean in)
		{
			live = in;
		}
		
		// genUp ()
		// increases a cell's age by one generation
		public void genUp ()
		{
			gen++;
		}
		
		// getGen ()
		// returns the cell's age
		public int getGen ()
		{
			return gen;
		}
		
		// printMe
		// if the cell is alive, print its age. Otherwise, print a blank space
		public void printMe ()
		{
			if (live)
				System.out.print(gen);
			else
				System.out.print(" ");
		}
	}