//**********************************************************************
// RulesPanel Class File
// The UI class that displays the Preferences dialog.
//******************************************************

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.util.Random;

//import java.awt.Rectangle;

public class rulesPanel extends JPanel
{
	public int counter;				// the counter number
	private JButton ok;		// the Increase button
	private JButton cancel;		// the Decrease button
	private JLabel label;			// the label. displays counter.
	
	private JCheckBox[] live;
	private JCheckBox[] dead;
	private JLabel liveLabel1;
	private JLabel liveLabel2;
	private JLabel deadLabel1;
	private JLabel deadLabel2;
	
	private JLabel presetLabel;
	
	private JComboBox rulesList;
	
	private boolean liveRules[];
	private boolean deadRules[];
	
	private JButton background;
	private JButton foreground;
	private JLabel colorLabel;
	private JLabel cellColorLabel;
	
	private JCheckBox showCellAge;
	private JCheckBox torus;
	
	private ruleSet[] builtInRules;
	
	private Color backgroundColor;
	private Color foregroundColor;
	int liveSeed;
	int deadSeed;
	LifePrefs myPrefs;
	
	board myBoard;
	
	boolean testGrid[][];
	boolean closed;
	
	JFrame myWindow;
	
	Box rulesCheck;
	Box liveChecks;
	Box deadChecks;
	Box buttonBox;
	Box colorBox;
	Box allRules;
	Box otherBox;
	Box popupBox;
	
	public static void main (String[] args)
	{

		LifePrefs temp = new LifePrefs();
		JFrame frame = new JFrame ("Preferences");
		LifePanel panel = new LifePanel();
				rulesPanel prefs = new rulesPanel(frame, temp, panel.dish);

		frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
		//ml = new MenuListener();
		//frame.setJMenuBar(ml.createMenuBar());
		
		frame.getContentPane().add(prefs);
		
		frame.pack();
		frame.setVisible(true);
	
	}
	
	class ruleSet
	{
		String name;
		int liveRuleMasks;
		int deadRuleMasks;
		
		public ruleSet (String newName, int live, int dead)
		{
			liveRuleMasks = live;
			deadRuleMasks = dead;
			name = newName;
		}
		
		public void setName (String newName)
		{
			name = newName;
		}
		
		public void setLive (int newLive)
		{
			liveRuleMasks = newLive;
		}
		
		public void setDead (int newDead)
		{
			liveRuleMasks = newDead;
		}
		
		public String getName ()
		{
			return name;
		}
		
		public int getLiveMask ()
		{
			return liveRuleMasks;
		}
		
		public int getDeadMask ()
		{
			return deadRuleMasks;
		}
	}

	public boolean[] seedToBools (int seed)
	{
		boolean ret[] = new boolean[9];
		int bits = seed;
		int placeCount = 0;
		while (bits > 0)
		{
			ret[placeCount] = ((bits & 1) == 1);
			placeCount++;
			bits = bits >> 1;
		}
		return ret;
	}
	
	public int boolsToSeed (boolean[] bools)
	{
		int seed = 0;
		int boolInt;
		for (int i = 0; i < bools.length; i++)
		{
			if (bools[i] == true)
				boolInt = 1;
			else boolInt = 0;
			seed += Math.pow(2,i) * boolInt;
		}
		return seed;
	}


	public boolean isClosed()
	{
		return closed;
	}
	
	public void setChecks(int newLiveSeed, int newDeadSeed)
	{
		liveRules = seedToBools(newLiveSeed);
		deadRules = seedToBools(newDeadSeed);
		for (int i = 0; i < 9; i++)
		{
			live[i].setSelected(liveRules[i]);
			dead[i].setSelected(deadRules[i]);
		}
	}
	
	public void searchChecks ()
	{
		boolean liveBool[] = new boolean[9];
		boolean deadBool[] = new boolean[9];
		for (int i = 0; i < 9; i++)
		{
			liveBool[i] = live[i].isSelected();
			deadBool[i] = dead[i].isSelected();
		}
		int deadCheck = boolsToSeed(deadBool);
		int liveCheck = boolsToSeed(liveBool);
		for (int i = 0; i < builtInRules.length; i++)
		{
			if (deadCheck == builtInRules[i].getDeadMask() && 
							liveCheck == builtInRules[i].getLiveMask())
			{
				rulesList.setSelectedIndex(i);
			}
		}
	}
	
	
	class RulesListener implements ActionListener 
	{
   	 public void actionPerformed(ActionEvent e)
		 {
        JComboBox cb = (JComboBox)e.getSource();
        String ruleName = (String)cb.getSelectedItem();
       	for (int i = 0; i < builtInRules.length; i++)
			{
				if (builtInRules[i].getName().compareTo(ruleName) == 0)
				{
					setChecks(builtInRules[i].getLiveMask(),builtInRules[i].getDeadMask());

				}
			}
  	  }
	}
	
	public rulesPanel (JFrame initWindow, LifePrefs inPrefs, board boardPtr)
	{
	
		int i;
		myWindow = initWindow;
		closed = false;
	
		builtInRules = new ruleSet[20];
		builtInRules[0] = new ruleSet("23/3 - Conway's Game of Life", 12, 8);
		builtInRules[1] = new ruleSet("/3 (stable) - almost everything is a spark", 0, 8);
		builtInRules[2] = new ruleSet("5678/35678 (chaotic) - diamonds, catastrophes", 480, 488);
		builtInRules[3] = new ruleSet("/2 (exploding) - (Seeds) phoenix, minimal", 0, 4);
		builtInRules[4] = new ruleSet("/234 (exploding) phoenix, lacey patterns", 0, 28);
		builtInRules[5] = new ruleSet("12345/3 (exploding) - maze-like designs", 62, 8);
		builtInRules[6] = new ruleSet("125/36 (chaotic) - Life-like 2x2 block rule", 38, 72);
		builtInRules[7] = new ruleSet("1357/1357 (exploding) - everything is a replicator", 170, 170);
		builtInRules[8] = new ruleSet("1358/357 (chaotic) - a balanced amoeba rule", 298, 168);
		builtInRules[9] = new ruleSet("23/36 (chaotic) 'HighLife' (has replicator)", 12, 72);
		builtInRules[10] = new ruleSet("235678/378 (exploding) - coagulations in chaos", 492, 392);
		builtInRules[11] = new ruleSet("238/357 (chaotic) - broken life", 268, 168);
		builtInRules[12] = new ruleSet("245/3 (chaotic) - different patterns to Conway's Life", 52, 8);
		builtInRules[13] = new ruleSet("245/368 (stable) - death plus puffers and ships", 52, 328);
		builtInRules[14] = new ruleSet("34/34 (exploding) - once thought to be stable", 24, 24);
		builtInRules[15] = new ruleSet("34678/3678 (exploding) \"Day & Night\"", 472, 456);
		builtInRules[16] = new ruleSet("4567/35678 (exploding) - fast growth", 240, 488);
		builtInRules[17] = new ruleSet("456/35678 - interesting variation of above", 112, 488);
		builtInRules[18] = new ruleSet("45678/3 (exploding) - slow coral growth", 496, 8);
		builtInRules[19] = new ruleSet("5/346 (stable) \"Long life\"", 32, 88);
		
		
		
		String rulesString[] = new String[builtInRules.length];
		for (i = 0; i < builtInRules.length; i++)
		{
			rulesString[i] = new String(builtInRules[i].getName());
		}
		
		
		myPrefs = inPrefs;
		myBoard = boardPtr;
		liveSeed = myPrefs.liveRules;
		deadSeed = myPrefs.deadRules;
		backgroundColor = myPrefs.backColor;
		foregroundColor = myPrefs.foreColor;

	
		
	
		//petList.addActionListener(this);
		
		//backgroundColor = Color.black;
		//foregroundColor = Color.white;
		
		Random rand = new Random();
	 	testGrid = new boolean[10][10];
		int r, c;
		for (r = 0; r < 10; r++)
			for (c= 0; c < 10; c++)
				if (rand.nextInt(4) == 1)
					testGrid[r][c] = true;
				else testGrid[r][c] = false;
					
		
		liveRules = new boolean[9];
		deadRules = new boolean[9];
		liveRules = seedToBools(liveSeed);
		deadRules = seedToBools(deadSeed);
		
		setLayout(new BorderLayout());
		
		 rulesCheck = new Box(BoxLayout.X_AXIS);
		 
		 allRules = new Box(BoxLayout.Y_AXIS);
		 liveChecks = new Box(BoxLayout.Y_AXIS);
		 deadChecks = new Box(BoxLayout.Y_AXIS);
		 buttonBox = new Box(BoxLayout.X_AXIS);
		 colorBox = new Box(BoxLayout.Y_AXIS);
		 otherBox = new Box(BoxLayout.Y_AXIS);
		 popupBox = new Box(BoxLayout.X_AXIS);
		 
		// allRules.setBorder(BorderFactory.createLineBorder(Color.black));
		
		//TitledBorder title;
		colorBox.setBorder(BorderFactory.createTitledBorder("Color"));
		allRules.setBorder(BorderFactory.createTitledBorder("Rules"));
		otherBox.setBorder(BorderFactory.createTitledBorder("Display"));
		//colorBox.setBorder(title);
		 
		liveLabel1 = new JLabel("Neighbors required");
		liveLabel2 = new JLabel("for a cell to survive:");
		deadLabel1 = new JLabel("Neighbors required");
		deadLabel2 = new JLabel("for a birth:");
		presetLabel = new JLabel("Preset: ");

		

		rulesList = new JComboBox(rulesString);
		rulesList.setSelectedIndex(myPrefs.selectedPreset);
		rulesList.addActionListener(new RulesListener());
		
		popupBox.add(presetLabel);
		popupBox.add(rulesList);
		allRules.add(popupBox);
		
		//allRules.add(presetLabel);
		//allRules.add(rulesList);
		
		allRules.add(rulesCheck);
		
		
		
		liveChecks.add(liveLabel1);
		liveChecks.add(liveLabel2);
		deadChecks.add(deadLabel1);
		deadChecks.add(deadLabel2);
	
		rulesCheck.add(liveChecks);
		rulesCheck.add(Box.createRigidArea(new Dimension(10,0)));
		rulesCheck.add(deadChecks);
		rulesCheck.add(Box.createRigidArea(new Dimension(10,0)));
		rulesCheck.add(Box.createHorizontalGlue());
		
		live = new JCheckBox[9];
		dead = new JCheckBox[9];
		
		colorLabel = new JLabel("Background color:");
		cellColorLabel = new JLabel("Cell color:");
		
		background = new JButton("Change...");
		foreground = new JButton("Change...");
		
		//background = new JButton("Change");
		background.addActionListener(new CounterListener());
		foreground.addActionListener(new CounterListener());
		
		colorBox.add(colorLabel);
		colorBox.add(Box.createRigidArea(new Dimension(20,35)));
		colorBox.add(background);
		colorBox.add(cellColorLabel);
		colorBox.add(Box.createRigidArea(new Dimension(20,35)));
		colorBox.add(foreground);
		colorBox.add(Box.createRigidArea(new Dimension(20,120)));
		
		showCellAge = new JCheckBox("Show cell age", myPrefs.showAge);
		otherBox.add(showCellAge);
		
		for (i = 0; i < 9; i++)
		{
			live[i] = new JCheckBox("" + i, liveRules[i]);
			dead[i] = new JCheckBox("" + i, deadRules[i]);
			live[i].addActionListener(new CounterListener());
			dead[i].addActionListener(new CounterListener());
			liveChecks.add(live[i]);
			deadChecks.add(dead[i]);
		}
		
		torus = new JCheckBox("Columbus Mode", myPrefs.torus);
		otherBox.add(torus);
		
		
		counter = 50;		// intialize counter
		
		// create label
		label = new JLabel ("Count: 50");
		
		// create OK and Cancel buttons
		ok = new JButton("OK");
		ok.addActionListener(new CounterListener());
		
		cancel = new JButton("Cancel");
		cancel.addActionListener(new CounterListener());
		

		
		// add buttons and label to panel
		add(allRules, BorderLayout.CENTER);
		add(otherBox, BorderLayout.EAST);
		add(label, BorderLayout.NORTH);
		add(colorBox, BorderLayout.WEST);
		buttonBox.add(Box.createHorizontalGlue());
		buttonBox.add(cancel);
		buttonBox.add(ok);
		add(buttonBox, BorderLayout.SOUTH);
	}
	
	public void paintComponent(Graphics page)
	{
		super.paintComponent(page);
		Rectangle bgRect, fgRect;
		bgRect = colorBox.getBounds();
		int top = (int)bgRect.getY();
		bgRect = colorLabel.getBounds();
		fgRect = cellColorLabel.getBounds();
		int bgTop = (int)bgRect.getY()+top + (int)bgRect.getHeight();
		int fgTop = (int)fgRect.getY()+top + (int)fgRect.getHeight();;
		page.setColor(backgroundColor);
		page.fillRect((int)bgRect.getX() + 10, bgTop + 5, 25,25);
		page.setColor(foregroundColor);
		page.fillRect((int)fgRect.getX() + 10, fgTop + 5, 25,25);
		page.setColor(Color.black);
		page.drawRect((int)bgRect.getX() + 9, bgTop + 4, 26,26);
		page.drawRect((int)fgRect.getX() + 9, fgTop + 4, 26,26);
		
		fgRect = foreground.getBounds();
		fgTop = (int)fgRect.getY() + top + (int)fgRect.getHeight() + 5;
				
		page.setColor(backgroundColor);
		page.fillRect(5,fgTop,100,100);
		page.setColor(Color.black);
		page.drawRect(4,fgTop-1,101,101);
		page.setColor(foregroundColor);
		int r, c;
		for (r = 0; r < 10; r++)
			for (c = 0; c < 10; c++)
				if (testGrid[r][c])
					page.fillRect(5 + (10 * c), fgTop + r * 10, 9, 9);
		
	}
	
	// the action listener for the buttons
	private class CounterListener implements ActionListener
	{
		public void actionPerformed (ActionEvent event)
		{
			// checks to see which button was pressed, and alters
			// counter accordingly
			if (event.getSource() == ok)
			{
				boolean liveRules[] = new boolean[9];
				boolean deadRules[] = new boolean[9];
				for (int i = 0; i < 9; i++)
				{
					liveRules[i] = live[i].isSelected();
					deadRules[i] = dead[i].isSelected();
				}
				closed = true;
				myWindow.hide();
				
				
				myPrefs.liveRules = boolsToSeed(liveRules);
				myPrefs.deadRules = boolsToSeed(deadRules);
				myPrefs.foreColor = foregroundColor;
				myPrefs.backColor = backgroundColor;
				myPrefs.showAge	= showCellAge.isSelected();
				myPrefs.torus = torus.isSelected();
				
				myPrefs.selectedPreset = rulesList.getSelectedIndex();
				
				myBoard.baseColor = foregroundColor;
				myBoard.initColors();
				myBoard.setCellActions (myPrefs.liveRules, myPrefs.deadRules);
				
				
			}
			else if (event.getSource() == cancel)
			{
				closed = true;
				myWindow.hide();
			}
			else if (event.getSource() == background)
			{
				Color temp = backgroundColor;
				temp = JColorChooser.showDialog(null, 
										"Background Color", backgroundColor);
				if (temp != null)
					backgroundColor = temp;
				repaint();
			}
			else if (event.getSource() == foreground)
			{
				Color temp = foregroundColor;
				temp = JColorChooser.showDialog(null, 
										"Cell Color", foregroundColor);
				if (temp != null)
					foregroundColor = temp;
				repaint();
			}
			for (int i = 0; i < 9; i++)
			{
				if (event.getSource() == live[i] || event.getSource() == dead[i])
				{
					searchChecks();
				}
			}
		}
	}
}